@@ -20,7 +20,7 @@ yes = C { f = (e h) } -- @Warning C {f = e h}
 yes = \ x -> (x && x) -- @Warning \x -> x && x
 no = \(x -> y) -> z
 yes = (`foo` (bar baz)) -- @Warning (`foo` bar baz)
-
+main = do f; (print x) -- @Warning do f print x
 
 -- type bracket reduction
 foo :: (Int -> Int) -> Int
@@ -47,6 +47,9 @@ yes = (a b $ c d) ++ e -- a b (c d) ++ e
 no = (f . g $ a) ++ e
 no = quickCheck ((\h -> cySucc h == succ h) :: Hygiene -> Bool)
 foo = (case x of y -> z; q -> w) :: Int
+
+-- backup fixity resolution
+main = do a += b . c; return $ a . b
 </TEST>
 -}
 
