@@ -1,4 +1,4 @@
-{-# LANGUAGE ViewPatterns, PatternGuards #-}
+{-# LANGUAGE ViewPatterns #-}
 
 {-
     Improve the structure of code
@@ -7,8 +7,8 @@
 yes x y = if a then b else if c then d else e -- yes x y ; | a = b ; | c = d ; | otherwise = e
 x `yes` y = if a then b else if c then d else e -- yes x y ; | a = b ; | c = d ; | otherwise = e
 no x y = if a then b else c
-foo b | c <- f b = c -- foo (f -> c) = c
-foo x y b z | c:cs <- f g b = c -- foo x y (f g -> c:cs) z = c
+-- foo b | c <- f b = c -- foo (f -> c) = c
+-- foo x y b z | c:cs <- f g b = c -- foo x y (f g -> c:cs) z = c
 foo b | c <- f b = c + b
 foo b | c <- f b = c where f = here
 foo b | c <- f b = c where foo = b
@@ -18,6 +18,13 @@ foo x = yes x x where yes x y = if a then b else if c then d else e -- yes x y ;
 foo x | otherwise = y -- foo x = y
 -- FIXME: #358 foo x = x + x where -- foo x = x + x
 foo x | a = b | True = d -- foo x | a = b ; | otherwise = d
+foo (Bar _ _ _ _) = x -- Bar{}
+foo (Bar _ x _ _) = x
+foo (Bar _ _) = x
+foo = case f v of _ -> x -- x
+foo = case v of v -> x -- x
+foo = case v of z -> z
+foo = case v of _ | False -> x
 </TEST>
 -}
 
@@ -26,11 +33,13 @@ module Hint.Structure where
 
 import Hint.Type
 import Util
-import Data.List
 
 
 structureHint :: DeclHint
-structureHint _ _ x = concatMap (uncurry hints . swap) $ asPattern x
+structureHint _ _ x =
+    concatMap (uncurry hints . swap) (asPattern x) ++
+    concatMap patHint (universeBi x) ++
+    concatMap expHint (universeBi x)
 
 
 hints :: (String -> Pattern -> Idea) -> Pattern -> [Idea]
@@ -38,6 +47,8 @@ hints gen (Pattern pat (UnGuardedRhs d bod) bind)
     | length guards > 2 = [gen "Use guards" $ Pattern pat (GuardedRhss d guards) bind]
     where guards = asGuards bod
 
+{-
+-- Do not suggest view patterns, they aren't something everyone likes sufficiently
 hints gen (Pattern pats (GuardedRhss _ [GuardedRhs _ [Generator _ pat (App _ op (view -> Var_ p))] bod]) bind)
     | Just i <- findIndex (=~= (toNamed p :: Pat_)) pats
     , p `notElem` (vars bod ++ vars bind)
@@ -46,6 +57,7 @@ hints gen (Pattern pats (GuardedRhss _ [GuardedRhs _ [Generator _ pat (App _ op
        Pattern (take i pats ++ [PParen an $ PViewPat an op pat] ++ drop (i+1) pats) (UnGuardedRhs an bod) bind]
     where
         decsBind = nub $ concatMap declBind $ childrenBi bind
+-}
 
 hints gen (Pattern pats (GuardedRhss _ [GuardedRhs _ [test] bod]) bind)
     | prettyPrint test `elem` ["otherwise","True"]
@@ -82,3 +94,18 @@ asPattern x = concatMap decl (universeBi x) ++ concatMap alt (universeBi x)
         match o@(Match a b pat rhs bind) = (Pattern pat rhs bind, \msg (Pattern pat rhs bind) -> warn msg o $ Match a b pat rhs bind)
         match o@(InfixMatch a p b ps rhs bind) = (Pattern (p:ps) rhs bind, \msg (Pattern (p:ps) rhs bind) -> warn msg o $ InfixMatch a p b ps rhs bind)
         alt o@(Alt a pat rhs bind) = [(Pattern [pat] (fromGuardedAlts rhs) bind, \msg (Pattern [pat] rhs bind) -> warn msg o $ Alt a pat (toGuardedAlts rhs) bind)]
+
+
+
+-- Should these hints be in the same module? They are less structure, and more about pattern matching
+-- Or perhaps the entire module should be renamed Pattern, since it's all about patterns
+patHint :: Pat_ -> [Idea]
+patHint o@(PApp _ name args) | length args >= 3 && all isPWildCard args = [warn "Use record patterns" o $ PRec an name []]
+patHint _ = []
+
+
+expHint :: Exp_ -> [Idea]
+expHint o@(Case _ _ [Alt _ PWildCard{} (UnGuardedAlt _ e) Nothing]) = [warn "Redundant case" o e]
+expHint o@(Case _ (Var _ x) [Alt _ (PVar _ y) (UnGuardedAlt _ e) Nothing])
+    | x =~= UnQual an y = [warn "Redundant case" o e]
+expHint _ = []
